import * as cdk from '@aws-cdk/core';
/**
 * @stability stable
 */
export interface GeneralRoleProps {
    /**
     * The name of the IAM policy for the IAM role.
     *
     * @default 'general-role-s3olap-policy'
     * @stability stable
     */
    readonly policyName?: string;
    /**
     * The name of the object Lambda access point, which will be the same as the S3 acceess point for the S3 bucket in the demostration.
     *
     * @default 'accessctl-s3olap-survey-results-unknown-pii'
     * @stability stable
     */
    readonly objectLambdaAccessPointName?: string;
    /**
     * The name of the IAM role.
     *
     * @default 'GeneralRole'
     * @stability stable
     */
    readonly iamRoleName?: string;
}
/**
 * The role that you are going to assume (switch role).
 *
 * Explores how the S3 Object Lambda works.
 *
 * @stability stable
 */
export declare class GeneralRole extends cdk.Construct {
    /**
     * The ARN of the IAM role.
     *
     * @stability stable
     */
    readonly roleArn: string;
    /**
     * The name of the IAM role.
     *
     * @stability stable
     */
    readonly roleName: string;
    /**
     * The unique string identifying the role.
     *
     * @stability stable
     */
    readonly roleId: string;
    /**
     * @stability stable
     */
    constructor(scope: cdk.Construct, id: string, props: GeneralRoleProps);
}
/**
 * @stability stable
 */
export interface AdminRoleProps {
    /**
     * The name of the IAM policy for the IAM role.
     *
     * @default 'admin-role-s3olap-policy'
     * @stability stable
     */
    readonly policyName?: string;
    /**
     * The name of the object Lambda access point, which will be the same as the S3 acceess point for the S3 bucket in the demostration.
     *
     * @default 'admin-s3olap-call-transcripts-known-pii'
     * @stability stable
     */
    readonly objectLambdaAccessPointName?: string;
    /**
     * The name of the IAM role.
     *
     * @default 'RedactionAdminRole'
     * @stability stable
     */
    readonly iamRoleName?: string;
}
/**
 * @stability stable
 */
export declare class AdminRole extends cdk.Construct {
    /**
     * The ARN of the IAM role.
     *
     * @stability stable
     */
    readonly roleArn: string;
    /**
     * The name of the IAM role.
     *
     * @stability stable
     */
    readonly roleName: string;
    /**
     * The unique string identifying the role.
     *
     * @stability stable
     */
    readonly roleId: string;
    /**
     * @stability stable
     */
    constructor(scope: cdk.Construct, id: string, props?: AdminRoleProps);
}
/**
 * @stability stable
 */
export interface BillingRoleProps {
    /**
     * The name of the IAM policy for the IAM role.
     *
     * @default 'billing-role-s3olap-policy'
     * @stability stable
     */
    readonly policyName?: string;
    /**
     * The name of the object Lambda access point, which will be the same as the S3 acceess point for the S3 bucket in the demostration.
     *
     * @default 'billing-s3olap-call-transcripts-known-pii'
     * @stability stable
     */
    readonly objectLambdaAccessPointName?: string;
    /**
     * The name of the IAM role.
     *
     * @default 'RedactionBillingRole'
     * @stability stable
     */
    readonly iamRoleName?: string;
}
/**
 * @stability stable
 */
export declare class BillingRole extends cdk.Construct {
    /**
     * The ARN of the IAM role.
     *
     * @stability stable
     */
    readonly roleArn: string;
    /**
     * The name of the IAM role.
     *
     * @stability stable
     */
    readonly roleName: string;
    /**
     * The unique string identifying the role.
     *
     * @stability stable
     */
    readonly roleId: string;
    /**
     * @stability stable
     */
    constructor(scope: cdk.Construct, id: string, props?: AdminRoleProps);
}
/**
 * @stability stable
 */
export interface CustSupportRoleProps {
    /**
     * The name of the IAM policy for the IAM role.
     *
     * @default 'customersupport-role-s3olap-policy'
     * @stability stable
     */
    readonly policyName?: string;
    /**
     * The name of the object Lambda access point, which will be the same as the S3 acceess point for the S3 bucket in the demostration.
     *
     * @default 'custsupport-s3olap-call-transcripts-known-pii'
     * @stability stable
     */
    readonly objectLambdaAccessPointName?: string;
    /**
     * The name of the IAM role.
     *
     * @default 'RedactionCustSupportRole'
     * @stability stable
     */
    readonly iamRoleName?: string;
}
/**
 * @stability stable
 */
export declare class CustSupportRole extends cdk.Construct {
    /**
     * The ARN of the IAM role.
     *
     * @stability stable
     */
    readonly roleArn: string;
    /**
     * The name of the IAM role.
     *
     * @stability stable
     */
    readonly roleName: string;
    /**
     * The unique string identifying the role.
     *
     * @stability stable
     */
    readonly roleId: string;
    /**
     * @stability stable
     */
    constructor(scope: cdk.Construct, id: string, props?: AdminRoleProps);
}
